#include "motif.h"

inMotif::inMotif() 
{
	num_constructed++;
	name.clear();
	marker = '*';
	regex.clear();
	sitemap.clear();
	bipartition.clear();
	sequence_template = false;
}

inMotif::inMotif(string prosite_name, string prosite_regex, size_t num_tips) 
{
	num_constructed++;
	name = prosite_name;
	marker = '*';
	regex = prosite_regex;
	sitemap.clear();
	bipartition.assign(true, num_tips);
	sequence_template = false;
}

inMotif::inMotif(inMotif *thisMotif, string& map, char mark) 
{
	num_constructed++;
	name = thisMotif->name;
	marker = mark;
	regex = thisMotif->regex;
	sitemap = map;
	bipartition.assign(thisMotif->bipartition.begin(), thisMotif->bipartition.end());
//	bipartition.assign(true, num_tips);
	sequence_template = false;
}

bool inMotif::isTemplate()
{
	if (isdigit(marker)) return true;
	else return false;
}

void inMotif::report() 
{
	cerr << "  name =        " << name << endl;
	cerr << "  marker =      " << marker << endl;
	cerr << "  bipartition = "; 
	for(vector<bool>::iterator it = bipartition.begin(); it != bipartition.end(); it++)
		cerr << (*it);
	cerr << endl;
	cerr << "  regex =       " << regex << endl;
	cerr << "  sitemap =     " << sitemap << endl;
}

vector<siteProperties*> inMotif::enumerateRegEx(TNode *node)
{
	size_t found, found2, found3;
	string site;
	list<string> return_sites;
	list<string> regex_sites;
	list<string> tmp;
	string my_stateCharacters;
	string reverse_sites;
	string x_length;
	int motif_min, motif_max;
	vector<siteProperties*> new_site_props;
	siteProperties *newSite;
	varSite *newVar;
	bool isTemplate;

	regex_sites = split(regex, "-");
	my_stateCharacters.clear();
	for (int i = 0; i < numStates; i++) my_stateCharacters += stateCharacters.at(i);
	return_sites.clear();

	for (list<string>::iterator it = regex_sites.begin(); it != regex_sites.end(); it++) {
		site.clear();
		switch ((*it).at(0)) {
		case '[':
			found = (*it).find("]", 1);
			if (found == string::npos) {
				cerr << "Regex position starts with '[', but has no ending ']'." << endl;
				exit(EXIT_FAILURE);
			}

			// Check characters in brackets. //
			site = (*it).substr(1,found-1);
			found = site.find_first_not_of(my_stateCharacters);
			if (found != string::npos) {
				cerr << "Unrecognized character in motif regular expression: " << (*it) << endl;
				exit(EXIT_FAILURE);
			}

			// Check to see if it is a multi-site, e.g., [LIV](3) //
			found2 = (*it).find("(", found);
			if (found2 != string::npos) {
				found3 = (*it).find(")", found2+1);
				x_length = (*it).substr(found2+1,found3-found2-1);
				int xlen = atoi(x_length.c_str());

				newVar = new varSite(motif_min, motif_max, node);

				for (int i = 0; i < xlen; i++) {
					newSite = new siteProperties(new_site_props, newVar, this, sequence_template);
					new_site_props.push_back(newSite);
					return_sites.push_back(site);
				}
			} else {
				newSite = new siteProperties(new_site_props, node->Generic_varSite(), this, sequence_template);
				new_site_props.push_back(newSite);
				return_sites.push_back(site);
			}
			break;
		case '{':
			found = (*it).find("}", 1);
			if (found == string::npos) {
				cerr << "Regex position starts with '{', but has no ending '}'." << endl;
				exit(EXIT_FAILURE);
			}
			reverse_sites = (*it).substr(1, found-1);
			found = reverse_sites.find_first_not_of(my_stateCharacters);
			if (found != string::npos) {
				cerr << "Unrecognized character in motif regular expression: " << (*it) << endl;
				exit(EXIT_FAILURE);
			}
			
			for (string::iterator it2 = my_stateCharacters.begin(); it2 != my_stateCharacters.end(); it2++) {
				found = reverse_sites.find((*it2));
				if (found == string::npos) site += (*it2);
			}

			found2 = (*it).find("(", found);
			if (found2 != string::npos) {
				found3 = (*it).find(")", found2+1);
				x_length = (*it).substr(found2+1,found3-found2-1);
				int xlen = atoi(x_length.c_str());

				newVar = new varSite(motif_min, motif_max, node);

				for (int i = 0; i < xlen; i++) {
					newSite = new siteProperties(new_site_props, newVar, this, sequence_template);
					new_site_props.push_back(newSite);
					return_sites.push_back(site);
				}
			} else {
				newSite = new siteProperties(new_site_props, node->Generic_varSite(), this, sequence_template);
				new_site_props.push_back(newSite);
				return_sites.push_back(site);
			}
			break;
		case 'x':
			if ((*it).size() == 1) {
				newSite = new siteProperties(new_site_props, node->Generic_varSite(), this, sequence_template);
				new_site_props.push_back(newSite);
				site = my_stateCharacters;	// just 'x'
				return_sites.push_back(my_stateCharacters);
			} else {
				tmp = split((*it), ",");
				if (tmp.size() > 1) {
					tmp.front().erase(0,2);
					tmp.back().erase(tmp.back().size()-1, 1);
					motif_min = atoi(tmp.front().c_str());
					motif_max = atoi(tmp.back().c_str());

					newVar = new varSite(motif_min, motif_max, node);

					// Fill out motif_min number of sites.
					for (size_t i = 0; i < motif_max; i++) {
						newSite = new siteProperties(new_site_props, newVar, this, sequence_template);
						new_site_props.push_back(newSite);
						return_sites.push_back(my_stateCharacters);
					}
				} else {
					size_t found2;
					(*it).erase(0,2);
					found2 = (*it).find(")");
					x_length = (*it).substr(0,found2);
					int xlen = atoi(x_length.c_str());

					newVar = new varSite(xlen, xlen, node);

					for (int i = 0; i < xlen; i++) {
						newSite = new siteProperties(new_site_props, newVar, this, sequence_template);
						new_site_props.push_back(newSite);
						return_sites.push_back(my_stateCharacters);
					}
				}
			}
			break;
		default:
			found = my_stateCharacters.find((*it).at(0));
			if (found != string::npos) {
				newSite = new siteProperties(new_site_props, node->Generic_varSite(), this, sequence_template);
				new_site_props.push_back(newSite);
				site.push_back((*it).at(0));
				return_sites.push_back(site);
			} else {
				cerr << "Unrecognized format for regular expression: " << (*it) << endl;
				exit(EXIT_FAILURE);
			}
			break;
		
		}
	}

	// Set bit string.
	vector<siteProperties*>::iterator it = new_site_props.begin();
	for (list<string>::iterator it3 = return_sites.begin(); it3 != return_sites.end(); it3++, it++) {
		bitset<20> regex_site;
		regex_site.set();
		for (string::iterator it4 = (*it3).begin(); it4 != (*it3).end(); it4++) {
			found = my_stateCharacters.find((*it4));
			if (found != string::npos) regex_site.flip(found);
		}
		regex_site.flip();
		(*it)->subst->setSiteBits(regex_site);
	}

	return new_site_props;
}

void inMotif::removeLastPosition() 
{
	// If last position is optional (e.g., [G>] in motif), and it is excluded from the motif,
	// this routine removes it from the inMotif prosite_library so that the regex size will
	// agree with the regex placed.
	list<string> regex_sites;
	string fixed_regex = "";
	regex_sites = split(regex, "-");
	regex_sites.pop_back();
	for (list<string>::iterator it = regex_sites.begin(); it != regex_sites.end(); it++) {
		fixed_regex += (*it);
		fixed_regex += "-";
	}
	fixed_regex.erase(fixed_regex.find_last_of("-"),1);
	regex = fixed_regex;
}

siteRegEx::siteRegEx(string regex, size_t sites, bool N_term, bool C_term, bool last_position_optional)
{
	num_constructed++;
	site_regex = regex;
	allowable_characters.clear();
	sites_occupied = sites;
	N_term_motif = N_term;
	C_term_motif = C_term;
	last_motif_site_optional = last_position_optional;
}

list<siteRegEx*> inMotif::parseRegEx()
{
	size_t found, found2, found3;
	string site;
	list<string> return_sites;
	list<string> regex_sites;
	list<string> tmp;
	string my_stateCharacters;
	string reverse_sites;
	string x_length;
	int motif_min, motif_max;
	bool isTemplate;
	list<siteRegEx*> return_list;
	siteRegEx *newSite;
	bool N_term = false, C_term = false, last_site_optional = false;

	found = regex.find("<", 0);
	if (found != string::npos) { 
		N_term = true; 
		regex.erase(found, 1); 
	}
	found = regex.find(">", 0);
	if (found != string::npos) {
		C_term = true;
		if (found != regex.size()-1) // ">" comes inside of [], e.g. [G>], meaning G is optional. //
			last_site_optional = true;
		regex.erase(found, 1);		
	}
	
	regex_sites = split(regex, "-");
	my_stateCharacters.clear();
	for (int i = 0; i < numStates; i++) my_stateCharacters += stateCharacters.at(i);
	return_sites.clear();
	return_list.clear();

	for (list<string>::iterator it = regex_sites.begin(); it != regex_sites.end(); it++) {
		site.clear();
		switch ((*it).at(0)) {
		case '[':
			newSite = new siteRegEx(*it, 1, N_term, C_term, last_site_optional);
			found = (*it).find("]", 1);
			if (found == string::npos) {
				cerr << "Regex position starts with '[', but has no ending ']'." << endl;
				exit(EXIT_FAILURE);
			}

			// Check characters in brackets. //
			site = (*it).substr(1,found-1);
			found = site.find_first_not_of(my_stateCharacters);
			if (found != string::npos) {
				cerr << "Unrecognized character in motif regular expression: " << (*it) << endl;
				exit(EXIT_FAILURE);
			}

			// Check to see if it is a multi-site, e.g., [LIV](3) //
			found2 = (*it).find("(", found);
			if (found2 != string::npos) {
				found3 = (*it).find(")", found2+1);
				x_length = (*it).substr(found2+1,found3-found2-1);
				int xlen = atoi(x_length.c_str());

				for (int i = 0; i < xlen; i++) newSite->allowable_characters.push_back(site);
				newSite->sites_occupied = xlen;
			} else {
				newSite->allowable_characters.push_back(site);
			}
			return_list.push_back(newSite);
			break;
		case '{':
			newSite = new siteRegEx(*it, 1, N_term, C_term, last_site_optional);
			found = (*it).find("}", 1);
			if (found == string::npos) {
				cerr << "Regex position starts with '{', but has no ending '}'." << endl;
				exit(EXIT_FAILURE);
			}
			reverse_sites = (*it).substr(1, found-1);
			found = reverse_sites.find_first_not_of(my_stateCharacters);
			if (found != string::npos) {
				cerr << "Unrecognized character in motif regular expression: " << (*it) << endl;
				exit(EXIT_FAILURE);
			}
			
			for (string::iterator it2 = my_stateCharacters.begin(); it2 != my_stateCharacters.end(); it2++) {
				found = reverse_sites.find((*it2));
				if (found == string::npos) site += (*it2);
			}

			found2 = (*it).find("(", found);
			if (found2 != string::npos) {
				found3 = (*it).find(")", found2+1);
				x_length = (*it).substr(found2+1,found3-found2-1);
				int xlen = atoi(x_length.c_str());

				for (int i = 0; i < xlen; i++) newSite->allowable_characters.push_back(site);
				newSite->sites_occupied = xlen;
			} else {
				newSite->allowable_characters.push_back(site);
			}
			return_list.push_back(newSite);
			break;
		case 'x':
			newSite = new siteRegEx(*it, 1, N_term, C_term, last_site_optional);
			if ((*it).size() == 1) {
				site = my_stateCharacters;	// just 'x'
				newSite->allowable_characters.push_back(site);
			} else {
				tmp = split((*it), ",");
				if (tmp.size() > 1) {
					tmp.front().erase(0,2);
					tmp.back().erase(tmp.back().size()-1, 1);
					motif_min = atoi(tmp.front().c_str());
					motif_max = atoi(tmp.back().c_str());

					newSite->sites_occupied = (double)rndu() * (motif_max-motif_min) + motif_min;
					// Fill out motif_max number of sites.
					for (size_t i = 0; i < motif_max; i++) newSite->allowable_characters.push_back(my_stateCharacters);
				} else {
					size_t found2;
					(*it).erase(0,2);
					found2 = (*it).find(")");
					x_length = (*it).substr(0,found2);
					int xlen = atoi(x_length.c_str());

					for (int i = 0; i < xlen; i++) newSite->allowable_characters.push_back(my_stateCharacters);
					newSite->sites_occupied = xlen;
				}
			}
			return_list.push_back(newSite);
			break;
		default:
			newSite = new siteRegEx(*it, 1, N_term, C_term, last_site_optional);
			found = my_stateCharacters.find((*it).at(0));
			if (found != string::npos) {
				site.push_back((*it).at(0));
				newSite->allowable_characters.push_back(site);
			} else {
				cerr << "Unrecognized format for regular expression: " << (*it) << endl;
				exit(EXIT_FAILURE);
			}
			return_list.push_back(newSite);
			break;
		
		}
	}

	// Set bit string.
	for (list<string>::iterator it3 = return_sites.begin(); it3 != return_sites.end(); it3++) {
		bitset<20> regex_site;
		regex_site.set();
		for (string::iterator it4 = (*it3).begin(); it4 != (*it3).end(); it4++) {
			found = my_stateCharacters.find((*it4));
			if (found != string::npos) regex_site.flip(found);
		}
		regex_site.flip();
	}

	return return_list;
}

bitset<20> inMotif::getRegExValues(size_t which_site)
{
	list<string> regex_sites;
	string my_stateCharacters;
	size_t at_current_site = 0;
	bitset<20> return_values;
	string site, reverse_sites;
	size_t found;
	list<string> tmp;
	string x_length;
	int motif_min, motif_max;
	bool done = false;

	regex_sites = split(regex, "-");
	my_stateCharacters.clear();
	for (int i = 0; i < numStates; i++) my_stateCharacters += stateCharacters.at(i);
	return_values.reset();

	for (list<string>::iterator it = regex_sites.begin(); it != regex_sites.end() && !done; it++) {
		site.clear();
		switch ((*it).at(0)) {
		case '[':
			at_current_site++;
			if (at_current_site == which_site) {

				found = (*it).find("]", 1);
				if (found == string::npos) {
					cerr << "Regex position starts with '[', but has no ending ']'." << endl;
					exit(EXIT_FAILURE);
				}
				site = (*it).substr(1,found-1);
				found = site.find_first_not_of(my_stateCharacters);
				if (found != string::npos) {
					cerr << "Unrecognized character in motif regular expression: " << (*it) << endl;
					exit(EXIT_FAILURE);
				} else {
					// Set return_values
					for (string::iterator sit = site.begin(); sit != site.end(); sit++) {
						found = my_stateCharacters.find_first_of(*sit);
						return_values.set(found);
					}
				}
				done = true;
			}
			break;
		case '{':
			at_current_site++;
			if (at_current_site == which_site) {
				found = (*it).find("}", 1);
				if (found == string::npos) {
					cerr << "Regex position starts with '{', but has no ending '}'." << endl;
					exit(EXIT_FAILURE);
				}
				reverse_sites = (*it).substr(1, found-1);
				found = reverse_sites.find_first_not_of(my_stateCharacters);
				if (found != string::npos) {
					cerr << "Unrecognized character in motif regular expression: " << (*it) << endl;
					exit(EXIT_FAILURE);
				}
				for (string::iterator it2 = my_stateCharacters.begin(); it2 != my_stateCharacters.end(); it2++) {
					found = reverse_sites.find((*it2));
					if (found == string::npos) site += (*it2);
				}
				for (string::iterator sit = site.begin(); sit != site.end(); sit++) {
					found = my_stateCharacters.find_first_of(*it);
					return_values.set(found);
				}
				done = true;
			}
			break;
		case 'x':
			if ((*it).size() == 1) {
				at_current_site++;
				if (at_current_site == which_site) {
					return_values.set();
					done = true;
				}
			} else {
				tmp = split((*it), ",");
				if (tmp.size() > 1) {
					tmp.front().erase(0,2);
					tmp.back().erase(tmp.back().size()-1, 1);
					motif_min = atoi(tmp.front().c_str());
					motif_max = atoi(tmp.back().c_str());

					if (motif_min == motif_max) {
						if (at_current_site + motif_min >= which_site) {
							return_values.set();
							done = true;
						} else at_current_site += motif_min;
					} else if (at_current_site + motif_max >= which_site) {
						done = true;
					} else at_current_site += motif_max;
				} else {
					size_t found2;
					(*it).erase(0,2);
					found2 = (*it).find(")");
					x_length = (*it).substr(0,found2);
					int xlen = atoi(x_length.c_str());

					if (at_current_site + xlen >= which_site) {
						return_values.set();
						done = true;
					} else at_current_site += xlen;
				}
			}
			break;
		default:
			at_current_site++;
			if (at_current_site == which_site) {
				found = my_stateCharacters.find((*it).at(0));
				if (found != string::npos) return_values.set(found);
				else {
					cerr << "Unrecognized format for regular expression: " << (*it) << endl;
					exit(EXIT_FAILURE);
				}
				done = true;
			}
			break;
		}
	}
	
	return return_values;
}

/////////// MOTIFSITE
motifSite::motifSite(motifSite *site_type, varSite *template_varSite, varSite *motif_varSite)
{
	num_constructed++;
	site_props.clear();
	site_props.push_back(new siteProperties(site_type, template_varSite, motif_varSite));
	activeProps = site_props.back();	// Sets activeProps to new site_props.
}

motifSite::motifSite(motifSite* site2copy)
{
	num_constructed++;
	if (site2copy == NULL) init();
	else copy(site2copy);
}

void motifSite::init() 
{
	site_props.clear();
	site_props.push_back(new siteProperties());
	activeProps = NULL;
}

void motifSite::Print_Nulls()
{
	if (activeProps == NULL) {
		cerr << "No active props";
	} else if (activeProps->indel == NULL) {
		cerr << "No indel props.";
	} else {
		if (activeProps->indel->del == NULL) cerr << "No del ";
		else {
			if (activeProps->indel->del->my_sequence_template_varSite == NULL) cerr << " del_mstv ";
			if (activeProps->indel->del->my_motif_varSite == NULL) cerr << " del_mmv ";
		}
		
		if (activeProps->indel->L_ins_ == NULL) cerr << "No L_ins_ ";
		else {
			if (activeProps->indel->L_ins_->my_sequence_template_varSite[LEFT] == NULL) cerr << " L_mstv[L] ";
			if (activeProps->indel->L_ins_->my_sequence_template_varSite[RIGHT] == NULL) cerr << " L_mstv[R] ";
			if (activeProps->indel->L_ins_->my_motif_varSite[LEFT] == NULL) cerr << " L_mmv[L] ";
			if (activeProps->indel->L_ins_->my_motif_varSite[RIGHT] == NULL) cerr << " L_mmv[R] ";
		}
		
		if (activeProps->indel->R_ins_ == NULL) cerr << "No R_ins_ ";
		else {
			if (activeProps->indel->R_ins_->my_sequence_template_varSite[LEFT] == NULL) cerr << " R_mstv[L] ";
			if (activeProps->indel->R_ins_->my_sequence_template_varSite[RIGHT] == NULL) cerr << " R_mstv[R] ";
			if (activeProps->indel->R_ins_->my_motif_varSite[LEFT] == NULL) cerr << " R_mmv[L] ";
			if (activeProps->indel->R_ins_->my_motif_varSite[RIGHT] == NULL) cerr << " R_mmv[R] ";
		}
	}
}

void motifSite::copy(motifSite* site2copy) 
{
	site_props.clear();
	for (list<siteProperties*>::iterator it = site2copy->site_props.begin(); it != site2copy->site_props.end(); it++) {
		site_props.push_back(new siteProperties(*it,this));
	}
}

void motifSite::setNode(TNode *node)
{
	my_node = node;
	for (list<siteProperties*>::iterator it = site_props.begin(); it != site_props.end(); it++)
		(*it)->setMotifSite(this);
}

void motifSite::setActiveProps(vector<bool>& this_bipartition, motifSite *prev, motifSite *anc, inClade *clade, bool last_site)
{
	int relationship;
	bool isTemplate;
	bool setTemplate=false, setMotif=false;
	short template_type = NO_RELATION, motif_type = NO_RELATION;
	siteProperties *template_props = NULL, *motif_props = NULL;
	siteProperties *anc_props = NULL;
	list<siteProperties*>::iterator it2;

	activeProps = new siteProperties(true,this);	// true means it is an activeProp build.

	if (anc != NULL) anc_props = anc->activeProps;
	for (list<siteProperties*>::iterator it = site_props.begin(); it != site_props.end(); it++) {
		if ( (*it)->fromMotif[TEMPLATE] != NULL) {	
			relationship = testRelation ((*it)->fromMotif[TEMPLATE]->bipartition, this_bipartition);
			if (relationship == EXACT) {
				if (setTemplate) {
					cerr << "Can only set one template per position per subtree." << endl;
					exit(EXIT_FAILURE);
				}
				setTemplate = true;
				template_type = EXACT;
				template_props = (*it);
			} else if (relationship == NODE_IS_ANC) {  // Is a template, but relationship is not exact.
				if (template_type != EXACT && template_type != NODE_IS_DES) {
					template_type = NODE_IS_ANC;
					template_props = (*it);
				}
			} else if (relationship == NODE_IS_DES) {
				if (template_type != EXACT) {
					template_type = NODE_IS_DES;
					template_props = (*it);
				}
			} else {		// NO_RELATIONSHIP
				// IGNORE
			}
		}
	}

	for (list<siteProperties*>::iterator it = site_props.begin(); it != site_props.end(); it++) {
		if ( (*it)->fromMotif[MOTIF] != NULL) {		// is a motif.
			relationship = testRelation ((*it)->fromMotif[MOTIF]->bipartition, this_bipartition);
			if (relationship == EXACT) {
				if (setMotif) {
					cerr << "Can only set one motif per position per subtree." << endl;
					exit(EXIT_FAILURE);
				}	
				setMotif = true;
				motif_type = EXACT;
				motif_props = (*it);
			} else if (relationship == NODE_IS_ANC) { 	// is a motif, relationship is not exact...
				if (motif_type != EXACT && motif_type != NODE_IS_DES) {
					motif_type = NODE_IS_ANC;
					motif_props = (*it);
				}
			} else if (relationship == NODE_IS_DES) {
				if (motif_type != EXACT) { 
					motif_type = NODE_IS_DES;
					motif_props = (*it);
				}
			} else {		// NO_RELATIONSHIP
				// IGNORE
			}
		}
	}

	if ( motif_props == NULL && template_props == NULL) {
		// Then it is a general props, treat it as so.
		activeProps->copy(site_props.front(),-1,-1,prev,NULL,last_site);
		activeProps->indel->my_siteProperties = site_props.front();
		activeProps->indel->setGeneral_varSites();

		if (last_site) {
			activeProps->indel->R_ins_ = new Insertion();
			activeProps->indel->R_ins_->my_sequence_template_varSite[LEFT] = activeProps->indel->R_ins_->my_sequence_template_varSite[RIGHT]
			= activeProps->indel->R_ins_->my_motif_varSite[LEFT] = activeProps->indel->R_ins_->my_motif_varSite[RIGHT]
			= activeProps->my_motifSite->my_node->unconstrained_varSite;
		}
	} else {
		// 	Found the properties for the activeProps, set values now.
		if (template_props) activeProps->copy(template_props,TEMPLATE,template_type,prev,anc_props,last_site);
		if (motif_props) activeProps->copy(motif_props,MOTIF,motif_type,prev,anc_props,last_site);
	}
}

void motifSite::deleteMerge(motifSite *prev, motifSite *next)
{
	// Briefly, variables are as such (with respect to this):
	//  prev      this      next
	//      \    /    \    /
	//       \  /      \  /
	//        1 2       3 4
	//       L_ins_    R_ins_

	if (next != NULL && prev != NULL) {
		next->activeProps->indel->L_ins_ = prev->activeProps->indel->R_ins_;
		next->activeProps->indel->L_ins_->my_sequence_template_varSite[RIGHT] = next->activeProps->indel->del->my_sequence_template_varSite;
		next->activeProps->indel->L_ins_->my_motif_varSite[RIGHT] = next->activeProps->indel->del->my_motif_varSite;
	} else {
		if (next == NULL) {
			activeProps->indel->L_ins_->my_sequence_template_varSite[LEFT]
			= activeProps->indel->L_ins_->my_sequence_template_varSite[RIGHT]
			= activeProps->indel->del->my_sequence_template_varSite;
			activeProps->indel->L_ins_->my_motif_varSite[LEFT]
			= activeProps->indel->L_ins_->my_motif_varSite[RIGHT]
			= activeProps->indel->del->my_motif_varSite;
		} else if (prev == NULL) cerr << "front end being deleted." << endl;
	}
	
	activeProps->indel->del->my_sequence_template_varSite->removeFromMember(activeProps->indel->del);
	activeProps->indel->del->my_motif_varSite->removeFromMember(activeProps->indel->del);
	deleteSiteObjects();
}

void motifSite::deleteSiteObjects()
{
	// Removing only activeProps. Need to also remove other regions, i.e., myMotif variables.
	bool aP = false;
	for (list<siteProperties*>::iterator it2 = site_props.begin(); it2 != site_props.end(); it2++) {
		// General sites become activeProps. If I run across a general site, I don't want to
		// delete the activeProps (b/c much non-hilarity ensues...)
		if ( (*it2)->indel->L_ins_ != activeProps->indel->L_ins_) {
			if ( it2 == site_props.begin() ) {
				delete (*it2)->indel->L_ins_;
			}
		} else aP = true;
		if ( (*it2)->indel->R_ins_ != activeProps->indel->R_ins_) 
			delete (*it2)->indel->R_ins_;
		if ( (*it2)->indel->del != activeProps->indel->del)
			delete (*it2)->indel->del;
		if ( (*it2)->subst != activeProps->subst)
			delete (*it2)->subst;
		if ( (*it2)->indel != activeProps->indel)
			delete (*it2)->indel;
		if ( (*it2) != activeProps)
			delete (*it2);
	}

	if ( !aP ) {
		delete activeProps->indel->R_ins_;
		delete activeProps->indel->del;
		delete activeProps->subst;
		delete activeProps->indel;
		delete activeProps;
	}
	site_props.clear();
}

/////// SITE PROPERTIES
siteProperties::siteProperties(motifSite *site_type, varSite *in_template_varSite, varSite *in_motif_varSite)
{
	num_constructed++;
	subst = new Substitution();		// Insertions have general substitution 
	indel = new Indel(in_template_varSite, in_motif_varSite);
	fromMotif[MOTIF] = site_type->activeProps->fromMotif[MOTIF];
	fromMotif[TEMPLATE] = site_type->activeProps->fromMotif[TEMPLATE];
}

siteProperties::siteProperties(bool activeProp /* = false */, motifSite *ms /* = NULL */)
{
	num_constructed++;
	subst = new Substitution();
	indel = new Indel(activeProp);
	fromMotif[MOTIF] = NULL;
	fromMotif[TEMPLATE] = NULL;
	if (ms != NULL) my_motifSite = ms;
	//else cerr << "motifSite null" << endl;
	ancestral = false;
}

siteProperties::siteProperties(vector<siteProperties*> prev_site, varSite *memberOfSite, inMotif *infromMotif, bool isTemplate) 
{
	num_constructed++;
	// Set subst props.
	subst = new Substitution();

	// Set up indel sites.
	indel = new Indel(prev_site);
	if (prev_site.empty()) {
		indel->L_ins_->setMembership(memberOfSite, isTemplate, RIGHT);
		indel->R_ins_->setMembership(memberOfSite, isTemplate, LEFT);
	} else {
		indel->R_ins_->setMembership(memberOfSite, isTemplate, LEFT);
		// Also need to set the previous site's membership to this spot.
		prev_site.back()->indel->L_ins_->setMembership(memberOfSite, isTemplate, RIGHT);
	}
	indel->del->setMembership(memberOfSite, isTemplate);

	// Record motif.
	if (isTemplate) {
		fromMotif[TEMPLATE] = infromMotif;
		fromMotif[MOTIF] = NULL;
	} else {
		fromMotif[MOTIF] = infromMotif;
		fromMotif[TEMPLATE] = NULL;
	}

	ancestral = false;
}

siteProperties::siteProperties(siteProperties* site2copy)
{
	num_constructed++;
//	if (site2copy->range != NULL)
//		range = new varSite(site2copy->range->min_length, site2copy->range->max_length, site2copy->range->from_inMotif);
//	else range = NULL;
//	current_num_sites = site2copy->current_num_sites;
//	site_bitstring.set();
//	site_bitstring &= site2copy->site_bitstring;

	vector<siteProperties*> pass_empty;
	pass_empty.clear();

	fromMotif[MOTIF] = site2copy->fromMotif[MOTIF];
	fromMotif[TEMPLATE] = site2copy->fromMotif[TEMPLATE];

	subst = new Substitution(site2copy->subst);
	if (fromMotif[TEMPLATE] != NULL) {
		indel = new Indel(site2copy->indel, NULL, true);	// NULL is ancestor.l
	} else if (fromMotif[MOTIF] != NULL) {
		indel = new Indel(site2copy->indel, NULL, false);
	} else {
		indel = new Indel(pass_empty);
//		indel = new Indel();
	}
}

siteProperties::siteProperties(siteProperties* site2copy, motifSite *anc)
{
	num_constructed++;
	vector<siteProperties*> pass_empty;
	pass_empty.clear();

	// The siteProperties is one of the sP's from anc.
	fromMotif[MOTIF] = site2copy->fromMotif[MOTIF];
	fromMotif[TEMPLATE] = site2copy->fromMotif[TEMPLATE];

	subst = new Substitution(site2copy->subst);
	if (fromMotif[TEMPLATE] != NULL) {
		indel = new Indel(site2copy->indel, anc, true);
	} else if (fromMotif[MOTIF] != NULL) {
		indel = new Indel(site2copy->indel, anc, false);
	} else {
//		indel = new Indel();
		indel = new Indel(pass_empty);
	}
}

void siteProperties::setMotifSite(motifSite *ms)
{
	my_motifSite = ms;
	indel->setSiteProperties(this);
}

void siteProperties::copy(siteProperties *stuff, short which, short type, motifSite *prev, siteProperties *anc, bool last_site)
{
	bitset<20> general;
	general.set();
	Indel *toPass = NULL;

	if (anc != NULL) toPass = anc->indel;

	// Substitutions are set ONLY in motif regions.
	if (which == TEMPLATE) {
		if (type != NO_RELATION) fromMotif[TEMPLATE] = stuff->fromMotif[TEMPLATE];
		if (type == EXACT) {
			indel->copy(stuff->indel, prev, true /*isTemplate*/, last_site, false/*isDescendant*/, toPass);	// True is the isTemplate
			indel->my_siteProperties = this;
			ancestral = false;
		} else if (type == NODE_IS_DES) {
			if (anc != NULL) {
				indel->copy(anc->indel, prev, true, last_site, true, toPass);
				indel->my_siteProperties = this;
			} else {
				cerr << "Should this be possible???" << endl;
				exit(EXIT_FAILURE);
			}
		} else if (type == NODE_IS_ANC) {
			indel->copy(stuff->indel, prev, true, last_site, false, toPass);
			indel->my_siteProperties = this;
		} else {
			// No relationship, do nothing.
		}
	} else if (which == MOTIF) {
		if (type != NO_RELATION) fromMotif[MOTIF] = stuff->fromMotif[MOTIF];
		if (type == EXACT) {
			subst->copy(stuff->subst);
			indel->copy(stuff->indel, prev, false, last_site, false, toPass);
			indel->my_siteProperties = this;
			ancestral = false;
		} else if (type == NODE_IS_DES) {
			subst->copy(anc->subst);
			indel->copy(anc->indel,prev, false,last_site, true, toPass);
			indel->my_siteProperties = this;
			ancestral = false;
		} else if (type == NODE_IS_ANC) {
			subst->setSiteBits(general);
			indel->copy(stuff->indel, prev, false, last_site, false, toPass);
			indel->my_siteProperties = this;
		} else {
			// No relationship, do nothing
		}
	} else {
		// This is for Pseudogene'd item. If pseudogene'd then we do not want to copy anything
		indel->copy(stuff->indel, prev, false, last_site, false, NULL);
		indel->my_siteProperties = this;
		subst->setSiteBits(general);
		ancestral = stuff->ancestral;
	}
}

void siteProperties::report() 
{
	cout << this << ": " << endl;
	
	cout << "  L_ins_ " << indel->L_ins_ << " mstv[LEFT]: ";
	if (indel->L_ins_->my_sequence_template_varSite[LEFT] == NULL) cout << "NULL";
	else cout << indel->L_ins_->my_sequence_template_varSite[LEFT]->min << " " << indel->L_ins_->my_sequence_template_varSite[LEFT]->max << "  ";
	cout << " mstv[RIGHT]: ";
	if (indel->L_ins_->my_sequence_template_varSite[RIGHT] == NULL) cout << "NULL";
	else cout << indel->L_ins_->my_sequence_template_varSite[RIGHT]->min << " " << indel->L_ins_->my_sequence_template_varSite[RIGHT]->max << "  ";
	cout << " mmv[LEFT]: ";
	if (indel->L_ins_->my_motif_varSite[LEFT] == NULL) cout << "NULL";
	else cout << indel->L_ins_->my_motif_varSite[LEFT]->min << " " << indel->L_ins_->my_motif_varSite[LEFT]->max << "  ";
	cout << " mmv[RIGHT]: ";
	if (indel->L_ins_->my_motif_varSite[RIGHT] == NULL) cout << "NULL";
	else cout << indel->L_ins_->my_motif_varSite[RIGHT]->min << " " << indel->L_ins_->my_motif_varSite[RIGHT]->max << "  ";
	cout << endl; 
	
	cout << "  R_ins_ " << indel->R_ins_ << " mstv[LEFT]: ";
	if (indel->R_ins_->my_sequence_template_varSite[LEFT] == NULL) cout << "NULL";
	else cout << indel->R_ins_->my_sequence_template_varSite[LEFT]->min << " " << indel->R_ins_->my_sequence_template_varSite[LEFT]->max << "  ";
	cout << " mstv[RIGHT]: ";
	if (indel->R_ins_->my_sequence_template_varSite[RIGHT] == NULL) cout << "NULL";
	else cout << indel->R_ins_->my_sequence_template_varSite[RIGHT]->min << " " << indel->R_ins_->my_sequence_template_varSite[RIGHT]->max << "  ";
	cout << " mmv[LEFT]: ";
	if (indel->R_ins_->my_motif_varSite[LEFT] == NULL) cout << "NULL";
	else cout << indel->R_ins_->my_motif_varSite[LEFT]->min << " " << indel->R_ins_->my_motif_varSite[LEFT]->max << "  ";
	cout << " mmv[RIGHT]: ";
	if (indel->R_ins_->my_motif_varSite[RIGHT] == NULL) cout << "NULL";
	else cout << indel->R_ins_->my_motif_varSite[RIGHT]->min << " " << indel->R_ins_->my_motif_varSite[RIGHT]->max << "  ";

	cout << endl << "  del " << indel->del << " ";
	if (indel->del->my_sequence_template_varSite == NULL) cout << "NULL";
	else cout << indel->del->my_sequence_template_varSite->min << " " << indel->del->my_sequence_template_varSite->max << " mmv: ";
	if (indel->del->my_motif_varSite == NULL) cout << "NULL";
	else cout << indel->del->my_motif_varSite->min << " " << indel->del->my_motif_varSite->max << " mmv: ";
	
	cout << endl << "  ";
	subst->report_bitset();


	cout << endl;
}

//////////// INSERTION, DELETION, SUBSTITUTION
Indel::Indel(vector<siteProperties*> prev_site /*set to null in decl.*/) 
{
	num_constructed++;
	if (prev_site.empty()) {
		L_ins_ = new Insertion();
		R_ins_ = new Insertion();
	} else {
		L_ins_ = prev_site.back()->indel->R_ins_;
		R_ins_ = new Insertion();
	}
	del = new Deletion();
}

Indel::Indel(varSite *in_template_varSite, varSite *in_motif_varSite)
{
	num_constructed++;
	L_ins_ = new Insertion(in_template_varSite, in_motif_varSite);
	R_ins_ = new Insertion(in_template_varSite, in_motif_varSite);
	del    = new Deletion(in_template_varSite, in_motif_varSite);
}

Indel::Indel(Indel *site2copy, motifSite *anc, bool isTemplate)
{
	num_constructed++;
	L_ins_ = new Insertion(site2copy->L_ins_, anc, isTemplate, NULL);
	R_ins_ = new Insertion(site2copy->R_ins_, anc, isTemplate, NULL);
	del = new Deletion(site2copy->del, anc, isTemplate);
}

Indel::Indel(bool activeProp) 
{
	num_constructed++;
	L_ins_ = new Insertion();
	if (!activeProp) R_ins_ = new Insertion();
	del = new Deletion();
}

void Indel::setGeneral_varSites()
{
	L_ins_->my_sequence_template_varSite[LEFT] = L_ins_->my_sequence_template_varSite[RIGHT]
	= L_ins_->my_motif_varSite[LEFT] = L_ins_->my_motif_varSite[RIGHT]
	= del->my_sequence_template_varSite = del->my_motif_varSite
	= my_siteProperties->my_motifSite->my_node->unconstrained_varSite;
}

void Indel::copy(Indel *site2copy, motifSite *prev_site, bool isTemplate, bool last_site, bool isDescendant, Indel *anc_indel)
{
	Insertion *anc_L_ins_ = NULL, *anc_R_ins_ = NULL;
	Deletion  *anc_del    = NULL;
	
	if (anc_indel != NULL) {
		if (anc_indel->L_ins_ != NULL) anc_L_ins_ = anc_indel->L_ins_;
		if (anc_indel->R_ins_ != NULL) anc_R_ins_ = anc_indel->R_ins_;
		if (anc_indel->del    != NULL) anc_del    = anc_indel->del;
	}

	L_ins_->copy(site2copy->L_ins_, isTemplate, isDescendant, anc_L_ins_);
	prev_site->activeProps->indel->R_ins_ = L_ins_;
	if (last_site) {
		R_ins_ = new Insertion();
		R_ins_->copy(site2copy->R_ins_,isTemplate, isDescendant, anc_R_ins_); // B/c otherwise NULL. maybe not smart to do.
		R_ins_->my_sequence_template_varSite[RIGHT] = R_ins_->my_sequence_template_varSite[LEFT];
		R_ins_->my_motif_varSite[RIGHT] = R_ins_->my_motif_varSite[LEFT];
	}
	del->copy(site2copy->del, isTemplate, isDescendant, anc_del);
}

void Indel::setSiteProperties(siteProperties *sp)
{
	my_siteProperties = sp;
}

void Indel::report()
{
	cout << "indel (des): " << setw(10) << this << endl;
	if (L_ins_ != NULL) {
		cout << "st L_ins_=" << setw(10) << L_ins_; 
		if (L_ins_->my_sequence_template_varSite[LEFT] != NULL) {
			cout << setw(10) << L_ins_->my_sequence_template_varSite[LEFT];
			cout << "(" << setw(2) << L_ins_->my_sequence_template_varSite[LEFT]->min; 
			cout << setw(6) << L_ins_->my_sequence_template_varSite[LEFT]->max << ")"; 
		} else cerr << "NULL";
		if (L_ins_->my_sequence_template_varSite[RIGHT] != NULL) {
			cout << setw(10) << L_ins_->my_sequence_template_varSite[RIGHT];
			cout << "(" << setw(2) << L_ins_->my_sequence_template_varSite[RIGHT]->min; 
			cout << setw(6) << L_ins_->my_sequence_template_varSite[RIGHT]->max << ")"; 
		} else cerr << "NULL";

		if (L_ins_->my_motif_varSite[LEFT] != NULL) {
			cout << "m L_ins_=" << setw(10) << L_ins_; 
			cout << setw(10) << L_ins_->my_motif_varSite[LEFT];
			cout << "(" << setw(2) << L_ins_->my_motif_varSite[LEFT]->min; 
			cout << setw(6) << L_ins_->my_motif_varSite[LEFT]->max << ")"; 
		} else cerr << "NULL";

		if (L_ins_->my_motif_varSite[RIGHT] != NULL) {
			cout << setw(10) << L_ins_->my_motif_varSite[RIGHT];
			cout << "(" << setw(2) << L_ins_->my_motif_varSite[RIGHT]->min; 
			cout << setw(6) << L_ins_->my_motif_varSite[RIGHT]->max << ")"; 
		} else cerr << "NULL";
	}

	if (R_ins_ != NULL) {
		cout << "st R_ins_=" << setw(10) << L_ins_; 
		if (R_ins_->my_sequence_template_varSite[LEFT] != NULL) {
			cout << setw(10) << R_ins_; 
			cout << setw(10) << R_ins_->my_sequence_template_varSite[LEFT];
			cout << "(" << setw(2) << R_ins_->my_sequence_template_varSite[LEFT]->min; 
			cout << setw(6) << R_ins_->my_sequence_template_varSite[LEFT]->max << ")"; 
		} else cerr << "NULL";

		if (R_ins_->my_sequence_template_varSite[RIGHT] != NULL) {
			cout << setw(10) << R_ins_->my_sequence_template_varSite[RIGHT];
			cout << "(" << setw(2) << R_ins_->my_sequence_template_varSite[RIGHT]->min; 
			cout << setw(6) << R_ins_->my_sequence_template_varSite[RIGHT]->max << ")"; 
			cout << endl;
		} else cerr << "NULL";

		if (R_ins_->my_motif_varSite[LEFT] != NULL) {
			cout << "m R_ins_=" << setw(10) << L_ins_; 
			cout << setw(10) << R_ins_; 
			cout << setw(10) << R_ins_->my_motif_varSite[LEFT];
			cout << "(" << setw(2) << R_ins_->my_motif_varSite[LEFT]->min; 
			cout << setw(6) << R_ins_->my_motif_varSite[LEFT]->max << ")"; 
		} else cerr << "NULL";

		if (L_ins_->my_motif_varSite[RIGHT] != NULL) {
			cout << setw(10) << R_ins_->my_motif_varSite[RIGHT];
			cout << "(" << setw(2) << R_ins_->my_motif_varSite[RIGHT]->min; 
			cout << setw(6) << R_ins_->my_motif_varSite[RIGHT]->max << ")"; 
			cout << endl;
		} else cerr << "NULL";
	}

	if (del != NULL) {
		cout << "st:m del=" << setw(10) << del; 
		if (del->my_sequence_template_varSite != NULL) {
			cout << setw(10) << del->my_sequence_template_varSite;
			cout << "(" << setw(2) << del->my_sequence_template_varSite->min;
			cout << setw(6) << del->my_sequence_template_varSite->max;
		} else cerr << "NULL";

		if (del->my_motif_varSite != NULL) {
			cout << setw(10) << del->my_motif_varSite;
			cout << "(" << setw(2) << del->my_motif_varSite->min;
			cout << setw(6) << del->my_motif_varSite->max;
		} else cerr << "NULL";
	}
}

Insertion::Insertion()
{
	num_constructed++;
	my_sequence_template_varSite[LEFT] = my_motif_varSite[LEFT] = NULL;
	my_sequence_template_varSite[RIGHT] = my_motif_varSite[RIGHT] = NULL;
	on_site_sequence_template_varSite = on_site_motif_varSite = NULL;
}

Insertion::Insertion(varSite *in_template_varSite, varSite *in_motif_varSite)
{
	num_constructed++;
	my_sequence_template_varSite[LEFT] = my_sequence_template_varSite[RIGHT] = in_template_varSite;
	my_motif_varSite[LEFT] = my_motif_varSite[RIGHT] = in_motif_varSite;
	if (in_template_varSite != NULL)
		if (in_template_varSite->min == 0) on_site_sequence_template_varSite = in_template_varSite;
		else on_site_sequence_template_varSite = NULL;
	if (in_motif_varSite != NULL)
		if (in_motif_varSite->min == 0) on_site_motif_varSite = in_motif_varSite;
		else on_site_motif_varSite = NULL;
}

Insertion::Insertion(Insertion *site2copy, motifSite *anc, bool isTemplate, Insertion *anc_copy)
{
	num_constructed++;
	// Copying ancestral, pointing to current struct.
	if (isTemplate) {
		if (site2copy->my_sequence_template_varSite[LEFT] == NULL) return;
		my_sequence_template_varSite[LEFT] = site2copy->my_sequence_template_varSite[LEFT];
		my_sequence_template_varSite[RIGHT] = site2copy->my_sequence_template_varSite[RIGHT];
		on_site_sequence_template_varSite = site2copy->on_site_sequence_template_varSite;
		if (anc_copy != NULL) {
			my_sequence_template_varSite[LEFT] = anc_copy->my_sequence_template_varSite[LEFT]->descendant_equiv;
			if (anc_copy->on_site_sequence_template_varSite != NULL)
				on_site_sequence_template_varSite = anc_copy->on_site_sequence_template_varSite->descendant_equiv;			
			else on_site_sequence_template_varSite = NULL;
			if (site2copy->my_sequence_template_varSite[RIGHT] == NULL) { cerr << "null." << endl; return; }
			my_sequence_template_varSite[RIGHT] = anc_copy->my_sequence_template_varSite[RIGHT]->descendant_equiv;
		}
	} else {
		if (site2copy->my_motif_varSite[LEFT] == NULL) return;
		my_motif_varSite[LEFT] = site2copy->my_motif_varSite[LEFT];
		my_motif_varSite[RIGHT] = site2copy->my_motif_varSite[RIGHT];
		on_site_motif_varSite = site2copy->on_site_motif_varSite;
		if (anc_copy != NULL) {
			my_motif_varSite[LEFT] = anc_copy->my_motif_varSite[LEFT];
			if (anc_copy->on_site_motif_varSite != NULL)
				on_site_motif_varSite = anc_copy->on_site_motif_varSite->descendant_equiv;
			else on_site_motif_varSite = NULL;
			if (site2copy->my_motif_varSite[RIGHT] == NULL) { cerr << "null." << endl; return; }
			my_motif_varSite[RIGHT] = anc_copy->my_motif_varSite[RIGHT];
		}
	}
}

void Insertion::copy(Insertion *site2copy, bool isTemplate, bool isDescendant, Insertion *anc_copy)
{
	if (isTemplate) {
		if (site2copy->my_sequence_template_varSite[LEFT] == NULL) return;
		if (isDescendant) {
			my_sequence_template_varSite[LEFT] = anc_copy->my_sequence_template_varSite[LEFT]->descendant_equiv;
			if (anc_copy->on_site_sequence_template_varSite != NULL)
				on_site_sequence_template_varSite = anc_copy->on_site_sequence_template_varSite->descendant_equiv;
			else on_site_sequence_template_varSite = NULL;
			if (anc_copy->my_sequence_template_varSite[RIGHT] != NULL)
				my_sequence_template_varSite[RIGHT] = anc_copy->my_sequence_template_varSite[RIGHT]->descendant_equiv;
		} else {
			my_sequence_template_varSite[LEFT] = site2copy->my_sequence_template_varSite[LEFT];
			my_sequence_template_varSite[RIGHT] = site2copy->my_sequence_template_varSite[RIGHT];
			on_site_sequence_template_varSite = site2copy->on_site_sequence_template_varSite;
		}
	} else {
		if (site2copy->my_motif_varSite[LEFT] == NULL) return;
		if (isDescendant) {
			my_motif_varSite[LEFT] = anc_copy->my_motif_varSite[LEFT]->descendant_equiv;
			if (anc_copy->on_site_motif_varSite != NULL) {
				on_site_motif_varSite = anc_copy->on_site_motif_varSite->descendant_equiv;
			} else on_site_motif_varSite = NULL;
			if (anc_copy->my_motif_varSite[RIGHT] != NULL)
				my_motif_varSite[RIGHT] = anc_copy->my_motif_varSite[RIGHT]->descendant_equiv;
		} else {
			my_motif_varSite[LEFT] = site2copy->my_motif_varSite[LEFT];
			my_motif_varSite[RIGHT] = site2copy->my_motif_varSite[RIGHT];		
			on_site_motif_varSite = site2copy->on_site_motif_varSite;
		}
	}
}

void Insertion::setMembership(varSite *memberOfSite, bool isTemplate, size_t side)
{
	if (isTemplate) {
		my_sequence_template_varSite[side] = memberOfSite;
		if (memberOfSite->min == 0) on_site_sequence_template_varSite = memberOfSite;
	} else {
		my_motif_varSite[side] = memberOfSite;
		if (memberOfSite->min == 0) on_site_motif_varSite = memberOfSite;
	}
}

bool Insertion::insertionAllowed(int size)
{
	if (on_site_sequence_template_varSite != NULL && on_site_motif_varSite != NULL) {
		if (on_site_sequence_template_varSite->insertion(size) && on_site_motif_varSite->insertion(size))
		return true;
	}
	
	if (on_site_sequence_template_varSite == NULL && on_site_motif_varSite != NULL) {
		// Motif will accept, but need to check if template left or right will accept an insertion.
		if (on_site_motif_varSite->insertion(size) && 		// Will on_site motif accept?
		    (my_sequence_template_varSite[LEFT]->insertion(size) || // WIll either st accept?
		     my_sequence_template_varSite[RIGHT]->insertion(size)))
			return true;
	} 
	
	if (on_site_sequence_template_varSite != NULL && on_site_motif_varSite == NULL) {
		if (on_site_sequence_template_varSite->insertion(size) &&
		    (my_motif_varSite[LEFT]->insertion(size) || my_motif_varSite[RIGHT]->insertion(size)))
			return true;
	}	// If both are NULL. don't need an else, since it is taken care of below.
		
	if ( my_sequence_template_varSite[LEFT]->insertion(size) && my_motif_varSite[LEFT]->insertion(size) ) 
		 return true;

	if ( my_sequence_template_varSite[RIGHT]->insertion(size) && my_motif_varSite[RIGHT]->insertion(size) ) 
		 return true;
	
	return false;
}

Deletion::Deletion()
{
	num_constructed++;
	my_sequence_template_varSite = my_motif_varSite = NULL;
}

Deletion::Deletion(varSite *in_template_varSite, varSite *in_motif_varSite)
{
	num_constructed++;
	my_sequence_template_varSite = in_template_varSite;
	my_motif_varSite = in_motif_varSite;
}

Deletion::Deletion(Deletion *site2copy, motifSite *anc, bool isTemplate)
{
	num_constructed++;
	if (anc != NULL) {
		if (isTemplate) my_sequence_template_varSite = site2copy->my_sequence_template_varSite->descendant_equiv;
		else my_motif_varSite = site2copy->my_motif_varSite->descendant_equiv;
	} else {
		if (isTemplate) my_sequence_template_varSite = site2copy->my_sequence_template_varSite;
		else my_motif_varSite = site2copy->my_motif_varSite;
	}
}

void Deletion::copy(Deletion *site2copy, bool isTemplate, bool isDescendant, Deletion *anc_copy)
{

	if (isTemplate) { 
		if (isDescendant) my_sequence_template_varSite = anc_copy->my_sequence_template_varSite->descendant_equiv;
		else my_sequence_template_varSite = site2copy->my_sequence_template_varSite;
	} else { 
		if (isDescendant) my_motif_varSite = anc_copy->my_motif_varSite->descendant_equiv;
		else my_motif_varSite = site2copy->my_motif_varSite;
	}
}

void Deletion::setMembership(varSite *memberOfSite, bool isTemplate)
{
	if (isTemplate) my_sequence_template_varSite = memberOfSite;
	else my_motif_varSite = memberOfSite;
}

bool Deletion::deletionAllowed(int size)
{
	if ( my_sequence_template_varSite->deletion(size) && my_motif_varSite->deletion(size) ) 
		 return true;

	return false;
}

////////////////////
////// SUBSTITUTION
////////////////////
Substitution::Substitution()
{
	num_constructed++;
	substitution_bitstring.set();
}

Substitution::Substitution(Substitution *site2copy)
{
	num_constructed++;
	substitution_bitstring.set();
	substitution_bitstring &= site2copy->substitution_bitstring;
}

bool Substitution::siteInvariable()
{
	if (substitution_bitstring.count() == 1) return true;
	else return false;
}

void Substitution::setInvariable(char residue)
{
	substitution_bitstring.reset();
	substitution_bitstring.set(residue);
}

void Substitution::copy(Substitution *site2copy)
{
	substitution_bitstring.set();
	substitution_bitstring &= site2copy->substitution_bitstring;
}

string Substitution::report_bitset()
{
	string report = "";
	for (int i = 0; i < numStates; i++)
		if (substitution_bitstring[i]) report += stateCharacters[i];

	return report;
}

void Substitution::setSiteBits(bitset<20> bitstring)
{
	substitution_bitstring.set();
	substitution_bitstring &= bitstring;
}

////////////////////
/////// VAR SITE
////////////////////
varSite::varSite (size_t minl, size_t maxl, TNode *my_node)
{
	num_constructed++;
	min = minl;
	max = maxl;
	members.clear();
	my_TNode = my_node;
	my_TNode->variable_region_list.push_back(this);
	descendant_equiv = NULL;
}

varSite::varSite (varSite *varSite2copy, TNode *my_node) 
{
	num_constructed++;
	min = varSite2copy->min;
	max = varSite2copy->max;
	members.clear();
	my_TNode = my_node;
	descendant_equiv = NULL;
	varSite2copy->descendant_equiv = this;
}

void varSite::add2Member (Deletion *newMember)
{
	members.push_back(newMember);
}

void varSite::removeFromMember (Deletion *thisMember)
{
	bool quit = false;
	
	for (list<Deletion*>::iterator it = members.begin(); !quit && it != members.end(); it++) {
		if (thisMember == (*it)) { 
			members.erase(it);
			quit = true;
		}
	}	
}

bool varSite::insertion(int size)
{

	if (members.size() + size > max) return false;
	if (max < members.size() + size) return false;

	return true;
}

bool varSite::deletion(int size)
{
	if (min == max) return false;
	if (min > members.size() - size) return false;

	return true;
}

bool varSite::isUnconstrained() 
{
	if (min == 0 && max == ISG_INT_MAX) return true;
	return false;
}
